--[[
Version 0.1.0
	Initial skeleton framework

--]]

local L = LibStub("AceLocale-3.0"):GetLocale("EasyDKP")
local AceAddon = LibStub("AceAddon-3.0")
EasyDKP = AceAddon:NewAddon("EasyDKP", "AceConsole-3.0", "AceEvent-3.0")
local REVISION = tonumber(("$Revision: 30 $"):match("%d+"))

EasyDKP.defaults = {
	profile = {
		enabled = true,
		debug = false,
		min_quality = 3,
		max_quality = 6,
		min_raiders = 1,
		chatoutput = "Raid", -- Output to raid by default
	},
}

EasyDKP.quality = {}
EasyDKP.quality[0] = { text = "Poor",      color = "Grey",   link=select(4, GetItemQualityColor(0))}
EasyDKP.quality[1] = { text = "Common",    color = "White",  link=select(4, GetItemQualityColor(1))}
EasyDKP.quality[2] = { text = "Uncommon",  color = "Green",  link=select(4, GetItemQualityColor(2))}
EasyDKP.quality[3] = { text = "Rare",      color = "Blue",   link=select(4, GetItemQualityColor(3))}
EasyDKP.quality[4] = { text = "Epic",      color = "Purple", link=select(4, GetItemQualityColor(4))}
EasyDKP.quality[5] = { text = "Legendary", color = "Orange", link=select(4, GetItemQualityColor(5))}
EasyDKP.quality[6] = { text = "Artifact",  color = "Gold",   link=select(4, GetItemQualityColor(6))}

EasyDKP.class = {}
EasyDKP.class[0] = { name = "Unknown", ticked = false, link = "|cff808080" }
EasyDKP.class[1] = { name = "Death Knight", ticked = false, link = "|cffc41f3b" }
EasyDKP.class[2] = { name = "Druid", ticked = false, link = "|cffff7d0a" }
EasyDKP.class[3] = { name = "Hunter", ticked = false, link = "|cffabd473" }
EasyDKP.class[4] = { name = "Mage", ticked = false, link = "|cff69ccf0" }
EasyDKP.class[5] = { name = "Paladin", ticked = false, link = "|cfff58cba" }
EasyDKP.class[6] = { name = "Priest", ticked = false, link = "|cffffffff" }
EasyDKP.class[7] = { name = "Rogue", ticked = false, link = "|cfffff569" }
EasyDKP.class[8] = { name = "Shaman", ticked = false, link = "|cff2459ff" }
EasyDKP.class[9] = { name = "Warlock", ticked = false, link = "|cff9482c9" }
EasyDKP.class[10] = { name = "Warrior", ticked = false, link = "|cffc79c6e" }

-----------------------------------------
-- Initialisation & Startup Routines
-----------------------------------------

function EasyDKP:OnInitialize()
	local AceConfigReg = LibStub("AceConfigRegistry-3.0")
	local AceConfigDialog = LibStub("AceConfigDialog-3.0")

	self.db = LibStub("AceDB-3.0"):New("EasyDKPDB", EasyDKP.defaults, "profile")
	LibStub("AceConfig-3.0"):RegisterOptionsTable("EasyDKP", self:GetOptions(), {"easydkp", "edkp"} )
	
	-- Add the options to blizzard frame (add them backwards so they show up in the proper order
	self.optionsFrame = AceConfigDialog:AddToBlizOptions("EasyDKP","EasyDKP")
	self.optionsFrame[L["About"]] = LibStub("LibAboutPanel").new("EasyDKP", "EasyDKP")
	self.db:RegisterDefaults(self.defaults)
	self:CreateFrame()
	
	local version = GetAddOnMetadata("EasyDKP","Version")
	self.version = ("EasyDKP v%s (r%s)"):format(version, REVISION)
	self:Print(self.version.." Loaded.")
end

function EasyDKP:OnLoad()
	self:CreateFrame()
end

function EasyDKP:OnDisable()
    -- Called when the addon is disabled
end

function EasyDKP:OnEnable()
	self:RegisterEvent("CHAT_MSG_LOOT")
end

function EasyDKP:CHAT_MSG_LOOT(event, chattext)
	if self.db.profile.enabled and (GetNumRaidMembers() >= self.db.profile.min_raiders or GetNumPartyMembers() >= self.db.profile.min_raiders) then 
		local start = string.find(chattext, L["%sreceive"])
		if start then
			start = start - 1
			local name = string.sub(chattext, 1, start)
			if (name == L["You"]) then
				name = UnitName("player")
			end
			local itemid = self:GetItemInfoFromLink(chattext)
			-- local itemName, itemString, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType = GetItemInfo(itemid) 
			local _, _, iRarity, iLevel = GetItemInfo(itemid)
			if(iRarity == nil) then
				iRarity = -1
			end
			if self.db.profile.debug then
				self:Print("decoded to itemid:"..itemid.." iRarity:"..iRarity.." for name:"..name..":")
			end
			-- 2 as raid default
			if(iRarity >= self.db.profile.min_quality and iRarity <= self.db.profile.max_quality) then
				local value = self:GetValue(itemid)
				if(value > 0) then				
					self:PopupQuestion(name, value)
				end
			end
		else
			if self.db.profile.debug then
				self:Print("didn't find string in chattext")
			end
		end
	end
end

function EasyDKP:ShowOptionsFrame()
	if not self.frame:IsVisible() then
		self.frame:Show()
	end
end

----------------------------
-- DKP listing functions
----------------------------

function EasyDKP:ListOptions(raidonly)
	local playercount = 0
	local playerlist = {}
	for key,val in pairs(dkp_info.members) do
		if val.class == "" then 
			val.class = "Unknown"
		end
		if not raidonly or (raidonly and UnitInRaid(val.name)) then
			if self:IncludeClass(val.class) then
				playerlist[val.name] = { ["name"] = val.name, ["class"] = val.class, ["dkp"] = val.dkp * 1, ["rank"] = val.rank}
			end
		end
	end
	if raidonly then 
		for i = 1, GetNumRaidMembers() do
			local name, _, _, _, class = GetRaidRosterInfo(i)
			if not playerlist[name] then
				if self:IncludeClass(val.class) then
					playerlist[name] = { ["name"] = name, ["class"] = class, ["dkp"] = 0, ["rank"] = "Not on DKP List"}
				end
			end
		end
	end	
	local sorttable = {}
	playercount = 0
	-- have to put it in a table with a numeric index for sorting to work???
	for key, val in pairs(playerlist) do
		playercount = playercount + 1
		table.insert(sorttable, playercount, { ["name"] = val.name, ["class"] = val.class, ["dkp"] = val.dkp, ["rank"] = val.rank })
	end
	table.sort(sorttable, sortDKP)
	self:Print(self.db.profile.chatoutput, L["Easy DKP list"])
	playercount = 0
	for key,val in ipairs(sorttable) do
		playercount = playercount + 1
		self:Print(self.db.profile.chatoutput, self:ClassColour(val.class)..playercount..": "..val.name..string.rep(".",14-string.len(val.name))..format("%.2f",val.dkp).." ("..val.rank..")")
	end
	if playercount == 0 then
		self:Print(self.db.profile.chatoutput, L["No dkp scores found matching chosen criteria"])
	end
end

function EasyDKP:PopupQuestion(name, value)
    local sQuestion = TEXT(format(L["Do you want to subtract %s from %s ?"], value, name))
    -- subtract dialog
    StaticPopupDialogs["POPUP_DIALOG"] = {
		text = sQuestion,
		button1 = L["Yes"],
		button2 = L["No"],
		OnAccept = function()
			EasyDKP:SubtractItem(name,value)
		end,
		OnCancel = function(_, reason)
--~ 			EasyDKP:CancelSubtract(name,value, reason)
		end,
		sound = "levelup2",
		timeout = 0,
		whileDead = 1,
		hideOnEscape = 1,
    }
    StaticPopup_Show ("POPUP_DIALOG")
end

function EasyDKP:SubtractItem(name,value)
	if self.db.profile.debug then		
		DEFAULT_CHAT_FRAME:AddMessage(L["EasyDKP subtract "]..name.." "..value.."|r")
	end
	self:Print(format(L["subtract %s %s"],name, value))
end

-----------------------------
--  Helper functions
-----------------------------

function sortDKP(el1, el2)
    return tonumber(el1["dkp"]) > tonumber(el2["dkp"])
end

function EasyDKP:IncludeClass(classname)
	local ticked = 0
	for i = 1, 10 do
		if self.class[i].ticked then
			if self.class[i].name == classname then
				return true
			end
			ticked = ticked + 1
		end
	end
	if ticked == 0 then
		return true -- return true if no classes ticked
	else
		return false
	end
end

function EasyDKP:ClassColour(class)
	for i = 1, 10 do
		if self.class[i].name == class then
			return self.class[i].link
		end
	end
	return "|cff808080"
end

-- process values for "specials"
function EasyDKP:GetValue(itemid)
	local value = 0
	itemid = tonumber(itemid)
	if self.db.profile.debug then
		self:Print("get value of "..itemid)
	end
	if itemid==29434 then -- Heroic Badge
		value = 0
	elseif itemid==40752 or itemid==40753 then -- Emblem of Heroism and Emblem of Valor
		value = 0
	elseif itemid==22450 or itemid==34057 then -- Void Crystal or Abyss Crystal - ignores disenchants
		value = 0
	elseif itemid==30183 then -- Nether Vortex
		value = 25 
	elseif itemid==34664 then -- Sunmote
		value = 25 
	elseif itemid>=29753 and itemid<=29767 then -- Tier 4 Tokens
		value = 120
	elseif itemid>=30236 and itemid<=30250 then -- Tier 5 Tokens
		value = 133
	else
		local sName, sLink, iRarity, iLevel, iMinLevel, sType, sSubType, iStackCount = GetItemInfo(itemid)
		value = iLevel
	end
	return value
end

function EasyDKP:GetItemInfoFromLink(link)
	local _, _, itemid, enchant, _, subid, itemname = string.find(link, "|Hitem:(%d+):")
	return tonumber(itemid or 0), tonumber(enchant or 0), tonumber(subid or 0), tonumber(itemname or 0)
end

function EasyDKP:ReturnStatus()
    local ret = L["Running: "]
    if self.db.profile.enabled then
	    ret = ret.."|c0000FF00"..L["Yes"].."|r"
    else
	    ret = ret.."|c00FF0000"..L["No"].."|r"
    end
    ret = ret.."\n"..L["Debug: "]
    if self.db.profile.debug then
	    ret = ret.."|c0000FF00"..L["Yes"].."|r"
    else
	    ret = ret.."|c00FF0000"..L["No"].."|r"
    end
    local range =  self.quality[self.db.profile.min_quality].link..self.quality[self.db.profile.min_quality].text.."|r to "..self.quality[self.db.profile.max_quality].link..self.quality[self.db.profile.max_quality].text
    ret = ret.."\n"..L["Range: "]..range
    return ret
end
